
module FP_reciprocal (

    input  logic [31:0] in,
    output logic [31:0] out

);

	assign out[31]    = in[31]   ;
	assign out[22: 0] = N2[22:0] ;
	assign out[30:23] = (D==9'b100000000)? 9'h102 - in[30:23] : 9'h101 - in[30:23] ;

	wire [31:0] D  ;
	wire [31:0] C1 ; //C1 = 48/17
	wire [31:0] C2 ; //C2 = 32/17
	wire [31:0] C3 ; //C3 = 2.0
	assign D  = {1'b0, 8'h80, in[22:0]} ;
	assign C1 = 32'h4034B4B5 ;
	assign C2 = 32'h3FF0F0F1 ;
	assign C3 = 32'h40000000 ;

	wire [31:0] N0 ;
	wire [31:0] N1 ;
	wire [31:0] N2 ;

	//Temporary connection wires
	wire [31:0] S0_2D_out       ;
	wire [31:0] S1_DN0_out      ;
	wire [31:0] S1_2min_DN0_out ;
	wire [31:0] S2_DN1_out      ;
	wire [31:0] S2_2minDN1_out  ;

	wire [31:0] S0_N0_in        ;
	assign S0_N0_in = {~S0_2D_out[31], S0_2D_out[30:0]} ;

	//S0
	FP_multiplier S0_2D
	(
		.a   ( C2        ),
		.b   ( D         ),
		.out ( S0_2D_out )
	);

	FP_adder S0_N0
	(
		.a   ( C1       ),
		.b   ( S0_N0_in ),
		.out ( N0       )
	);

	//S1
	FP_multiplier S1_DN0
	(
		.a   ( D          ),
		.b   ( N0         ),
		.out ( S1_DN0_out )
	);

	FP_adder S1_2minDN0
	(
		.a   ( C3                                  ),
		.b   ( {~S1_DN0_out[31], S1_DN0_out[30:0]} ),
		.out ( S1_2min_DN0_out                     )
	);

	FP_multiplier S1_N1
	(
		.a   ( N0              ),
		.b   ( S1_2min_DN0_out ),
		.out ( N1              )
	);

	//S2
	FP_multiplier S2_DN1
	(
		.a   ( D          ),
		.b   ( N1         ),
		.out ( S2_DN1_out )
	);

	FP_adder S2_2minDN1
	(
		.a   ( C3                                  ),
		.b   ( {~S2_DN1_out[31], S2_DN1_out[30:0]} ),
		.out ( S2_2minDN1_out                      )
	);

	FP_multiplier S2_N2
	(
		.a   ( N1             ),
		.b   ( S2_2minDN1_out ),
		.out ( N2             )
	);

endmodule
